"""Const for iOS."""

DOMAIN = "ios"

CONF_ACTION_NAME = "name"
CONF_ACTION_BACKGROUND_COLOR = "background_color"
CONF_ACTION_LABEL = "label"
CONF_ACTION_LABEL_COLOR = "color"
CONF_ACTION_LABEL_TEXT = "text"
CONF_ACTION_ICON = "icon"
CONF_ACTION_ICON_COLOR = "color"
CONF_ACTION_ICON_ICON = "icon"
CONF_ACTIONS = "actions"
